<?php namespace Myriad\Carmen\Contracts;

use GuzzleHttp\Client as Guzzle;
use GuzzleHttp\Exception\ServerException;
use Myriad\Carmen\CarmenConfiguration;

interface Carmen {

  /**
   * Create a new instance of Carmen
   *
   * @param \Myriad\Carmen\CarmenConfiguration $config
   * @param \GuzzleHttp\Client $guzzle
   */
  public function __construct(CarmenConfiguration $config, Guzzle $guzzle);

  /**
   * Get a lat and lng from zipcode
   *
   * @param string $zip the zip to query on
   *
   * @return array latlng
   * @throws ServerException
   */
  public function getLatLngFromZip($zip);

  /**
   * Get the distance between two latlng
   *
   * @param array $latlng the first latlng to query on
   * @param array $otherLatlng the second latlng to query on
   * @param string $units the type of units to return
   *
   * @return double distance
   */
  public function getDistanceBetween(array $latlng, array $otherLatlng, $unit);

  /**
   * Get a collection of objects within a radius
   *
   * @param string $latlng the latlng at center of radius
   * @param string $radius the radius of objects to return
   * @param string $units the units of the radius
   * @param string $table the name of the table of objects
   * @param string $latColumn the lat column of the table
   * @param string $lngColumn the lng column of the table
   *
   * @return collection objects
   */
  public function getWithin($latlng, $radius, $unit, $table, $latColumn='lat', $lngColumn='lng');

  /**
   * Get a collection of objects outside a radius
   *
   * @param string $latlng the latlng at center of radius
   * @param string $radius the radius of objects to return
   * @param string $units the units of the radius
   * @param string $table the name of the table of objects
   * @param string $latColumn the lat column of the table
   * @param string $lngColumn the lng column of the table
   *
   * @return collection objects
   */
  public function getOutside($latlng, $radius, $unit, $table, $latColumn='lat', $lngColumn='lng');

  /**
   * Get all US postal codes by Fips code lookup
   * @param array $fipsCodes These can be the 5-digit county or 2 digit state.
   * @return array An array of postal codes.
   */
  public function getUSPostalCodesByFipsCodes($fipsCodes);

  /**
   * Get the postal code object for code and country.
   * @param $postalCode
   * @param null $countryCode
   * @return mixed
   */
  public function getPostalCode($postalCode, $countryCode = null);

  /**
   * Get the Carmen configuration
   *
   * @return CarmenConfiguration
   */
  public function getConfiguration();

}